@@ -86,14 +86,16 @@ private Q_SLOTS:
         {
             LocationRequest req;
             req.setCoordinate(-52.5, -13.5);
-            QCOMPARE(req.cacheKey(), QLatin1String("-52500000x-13500000"));
+            QCOMPARE(req.cacheKey(), QLatin1String("73be02317c4a7a43aff31f3186b18c22c6466250"));
+            req.setTypes(Location::RentedVehicle|Location::RentedVehicleStation);
+            QCOMPARE(req.cacheKey(), QLatin1String("4dfaecadb1a7d03106fe585b119948c158509807"));
         }
         {
             LocationRequest req;
             req.setName(QStringLiteral("Randa"));
-            QCOMPARE(req.cacheKey(), QLatin1String("8fb4b1073d1c288ae089ce8eb7ad0c16e478e919"));
+            QCOMPARE(req.cacheKey(), QLatin1String("c4053bc0f4582f2977f3e43cad07b6587e6b6935"));
             req.setName(QStringLiteral("RANDA"));
-            QCOMPARE(req.cacheKey(), QLatin1String("8fb4b1073d1c288ae089ce8eb7ad0c16e478e919"));
+            QCOMPARE(req.cacheKey(), QLatin1String("c4053bc0f4582f2977f3e43cad07b6587e6b6935"));
         }
     }
 };
