@@ -18,7 +18,9 @@
 #include "vehiclelayoutreply.h"
 #include "datatypes/attributionutil_p.h"
 #include "datatypes/backend.h"
+#include "datatypes/backend_p.h"
 #include "datatypes/disruption.h"
+#include "datatypes/json_p.h"
 #include "datatypes/platform.h"
 #include "datatypes/vehicle.h"
 #include "geo/geojson_p.h"
@@ -73,7 +75,7 @@ public:
     template <typename T>
     static std::unique_ptr<AbstractBackend> loadNetwork(const QJsonObject &obj);
 
-    template <typename RequestT> bool shouldSkipBackend(const AbstractBackend *backend, const RequestT &req) const;
+    template <typename RequestT> bool shouldSkipBackend(const Backend &backend, const RequestT &req) const;
 
     void resolveLocation(LocationRequest &&locReq, const AbstractBackend *backend, const std::function<void(const Location &loc)> &callback);
     bool queryJourney(const AbstractBackend *backend, const JourneyRequest &req, JourneyReply *reply);
@@ -85,9 +87,8 @@ public:
 
     Manager *q = nullptr;
     QNetworkAccessManager *m_nam = nullptr;
-    std::vector<std::unique_ptr<AbstractBackend>> m_backends;
+    std::vector<Backend> m_backends;
     std::vector<Attribution> m_attributions;
-    std::vector<Backend> m_backendMetaData;
 
     // we store both explicitly to have a third state, backends with the enabled state being the "default" (whatever that might eventually be)
     QStringList m_enabledBackends;
@@ -98,7 +99,7 @@ public:
     bool m_backendsEnabledByDefault = true;
 
 private:
-    bool shouldSkipBackend(const AbstractBackend *backend) const;
+    bool shouldSkipBackend(const Backend &backend) const;
 };
 }
 
@@ -113,21 +114,6 @@ QNetworkAccessManager* ManagerPrivate::nam()
     return m_nam;
 }
 
-static QString translatedValue(const QJsonObject &obj, const QString &key)
-{
-    auto languageWithCountry = QLocale().name();
-    auto it = obj.constFind(key + QLatin1Char('[') + languageWithCountry + QLatin1Char(']'));
-    if (it != obj.constEnd()) {
-        return it.value().toString();
-    }
-    const auto language = languageWithCountry.midRef(0, languageWithCountry.indexOf(QLatin1Char('_')));
-    it = obj.constFind(key + QLatin1Char('[') + language + QLatin1Char(']'));
-    if (it != obj.constEnd()) {
-        return it.value().toString();
-    }
-    return obj.value(key).toString();
-}
-
 
 void ManagerPrivate::loadNetworks()
 {
@@ -154,24 +140,15 @@ void ManagerPrivate::loadNetworks()
                 m_attributions.push_back(net->attribution());
             }
 
-            Backend metaData;
-            metaData.setIdentifier(net->backendId());
-            const auto jsonMetaData = doc.object().value(QLatin1String("KPlugin")).toObject();
-            metaData.setName(translatedValue(jsonMetaData, QStringLiteral("Name")));
-            metaData.setDescription(translatedValue(jsonMetaData, QStringLiteral("Description")));
-            metaData.setIsSecure(net->capabilities() & AbstractBackend::Secure);
-
-            m_backendMetaData.push_back(std::move(metaData));
-            m_backends.push_back(std::move(net));
+            auto b = BackendPrivate::fromJson(doc.object());
+            BackendPrivate::setImpl(b, std::move(net));
+            m_backends.push_back(std::move(b));
         } else {
             qCWarning(Log) << "Failed to load public transport network configuration config:" << it.fileName();
         }
     }
 
     std::stable_sort(m_backends.begin(), m_backends.end(), [](const auto &lhs, const auto &rhs) {
-        return lhs->backendId() < rhs->backendId();
-    });
-    std::stable_sort(m_backendMetaData.begin(), m_backendMetaData.end(), [](const auto &lhs, const auto &rhs) {
         return lhs.identifier() < rhs.identifier();
     });
 
@@ -244,16 +221,6 @@ static void applyBackendOptions(AbstractBackend *backend, const QMetaObject *mo,
         }
     }
 
-    const auto coverageData = obj.value(QLatin1String("coverage")).toObject();
-    for (const auto &coverageType : { "anyCoverage", "regularCoverage", "realtimeCoverage" }) {
-        const auto coverage = coverageData.value(QLatin1String(coverageType)).toObject();
-        if (coverage.empty()) {
-            continue;
-        }
-        const auto poly = GeoJson::readOuterPolygon(coverage.value(QLatin1String("area")).toObject());
-        backend->setGeoFilter(poly);
-    }
-
     const auto attrObj = obj.value(QLatin1String("attribution")).toObject();
     const auto attr = Attribution::fromJson(attrObj);
     backend->setAttribution(attr);
@@ -282,20 +249,20 @@ template<typename T> std::unique_ptr<AbstractBackend> ManagerPrivate::loadNetwor
     return backend;
 }
 
-bool ManagerPrivate::shouldSkipBackend(const AbstractBackend *backend) const
+bool ManagerPrivate::shouldSkipBackend(const Backend &backend) const
 {
-    if (!backend->hasCapability(AbstractBackend::Secure) && !m_allowInsecure) {
-        qCDebug(Log) << "Skipping insecure backend:" << backend->backendId();
+    if (!backend.isSecure() && !m_allowInsecure) {
+        qCDebug(Log) << "Skipping insecure backend:" << backend.identifier();
         return true;
     }
-    return !q->isBackendEnabled(backend->backendId());
+    return !q->isBackendEnabled(backend.identifier());
 }
 
 template <typename RequestT>
-bool ManagerPrivate::shouldSkipBackend(const AbstractBackend *backend, const RequestT &req) const
+bool ManagerPrivate::shouldSkipBackend(const Backend &backend, const RequestT &req) const
 {
-    if (!req.backendIds().isEmpty() && !req.backendIds().contains(backend->backendId())) {
-        //qCDebug(Log) << "Skipping backend" << backend->backendId() << "due to explicit request";
+    if (!req.backendIds().isEmpty() && !req.backendIds().contains(backend.identifier())) {
+        //qCDebug(Log) << "Skipping backend" << backend.identifier() << "due to explicit request";
         return true;
     }
     return shouldSkipBackend(backend);
@@ -354,15 +321,6 @@ static Location::Types locationTypesForJourneyRequest(const JourneyRequest &req)
 
 bool ManagerPrivate::queryJourney(const AbstractBackend* backend, const JourneyRequest &req, JourneyReply *reply)
 {
-    if (shouldSkipBackend(backend, req)) {
-        return false;
-    }
-    if (backend->isLocationExcluded(req.from()) && backend->isLocationExcluded(req.to())) {
-        qCDebug(Log) << "Skipping backend based on location filter:" << backend->backendId();
-        return false;
-    }
-    reply->addAttribution(backend->attribution());
-
     auto cache = Cache::lookupJourney(backend->backendId(), req.cacheKey());
     switch (cache.type) {
         case CacheHitType::Negative:
@@ -429,19 +387,6 @@ bool ManagerPrivate::queryJourney(const AbstractBackend* backend, const JourneyR
 
 bool ManagerPrivate::queryStopover(const AbstractBackend *backend, const StopoverRequest &req, StopoverReply *reply)
 {
-    if (shouldSkipBackend(backend, req)) {
-        return false;
-    }
-    if (req.mode() == StopoverRequest::QueryArrival && (backend->capabilities() & AbstractBackend::CanQueryArrivals) == 0) {
-        qCDebug(Log) << "Skipping backend due to not supporting arrival queries:" << backend->backendId();
-        return false;
-    }
-    if (backend->isLocationExcluded(req.stop())) {
-        qCDebug(Log) << "Skipping backend based on location filter:" << backend->backendId();
-        return false;
-    }
-    reply->addAttribution(backend->attribution());
-
     auto cache = Cache::lookupDeparture(backend->backendId(), req.cacheKey());
     switch (cache.type) {
         case CacheHitType::Negative:
@@ -562,9 +507,27 @@ JourneyReply* Manager::queryJourney(const JourneyRequest &req) const
 
     // first time/direct query
     if (req.contexts().empty()) {
-        for (const auto &backend : d->m_backends) {
-            if (d->queryJourney(backend.get(), req, reply)) {
-                ++pendingOps;
+        QSet<QString> triedBackends;
+        bool foundNonGlobalCoverage = false;
+        for (const auto coverageType : { CoverageArea::Realtime, CoverageArea::Regular, CoverageArea::Any }) {
+            for (const auto &backend: d->m_backends) {
+                if (triedBackends.contains(backend.identifier()) || d->shouldSkipBackend(backend, req)) {
+                    continue;
+                }
+                const auto coverage = backend.coverageArea(coverageType);
+                if (coverage.isEmpty() || (!coverage.coversLocation(req.from()) && !coverage.coversLocation(req.to()))) {
+                    continue;
+                }
+                triedBackends.insert(backend.identifier());
+                foundNonGlobalCoverage |= !coverage.isGlobal();
+
+                if (d->queryJourney(BackendPrivate::impl(backend), req, reply)) {
+                    ++pendingOps;
+                }
+            }
+
+            if (pendingOps && foundNonGlobalCoverage) {
+                break;
             }
         }
 
@@ -618,9 +581,31 @@ StopoverReply* Manager::queryStopover(const StopoverRequest &req) const
 
     // first time/direct query
     if (req.contexts().empty()) {
-        for (const auto &backend : d->m_backends) {
-            if (d->queryStopover(backend.get(), req, reply)) {
-                ++pendingOps;
+        QSet<QString> triedBackends;
+        bool foundNonGlobalCoverage = false;
+        for (const auto coverageType : { CoverageArea::Realtime, CoverageArea::Regular, CoverageArea::Any }) {
+            for (const auto &backend: d->m_backends) {
+                if (triedBackends.contains(backend.identifier()) || d->shouldSkipBackend(backend, req)) {
+                    continue;
+                }
+                if (req.mode() == StopoverRequest::QueryArrival && (BackendPrivate::impl(backend)->capabilities() & AbstractBackend::CanQueryArrivals) == 0) {
+                    qCDebug(Log) << "Skipping backend due to not supporting arrival queries:" << backend.identifier();
+                    continue;
+                }
+                const auto coverage = backend.coverageArea(coverageType);
+                if (coverage.isEmpty() || !coverage.coversLocation(req.stop())) {
+                    continue;
+                }
+                triedBackends.insert(backend.identifier());
+                foundNonGlobalCoverage |= !coverage.isGlobal();
+
+                if (d->queryStopover(BackendPrivate::impl(backend), req, reply)) {
+                    ++pendingOps;
+                }
+            }
+
+            if (pendingOps && foundNonGlobalCoverage) {
+                break;
             }
         }
 
@@ -670,32 +655,42 @@ LocationReply* Manager::queryLocation(const LocationRequest &req) const
         return reply;
     }
 
-    for (const auto &backend : d->m_backends) {
-        if (d->shouldSkipBackend(backend.get(), req)) {
-            continue;
-        }
-        if (req.hasCoordinate() && backend->isLocationExcluded(req.location())) {
-            qCDebug(Log) << "Skipping backend based on location filter:" << backend->backendId();
-            continue;
-        }
-        reply->addAttribution(backend->attribution());
+    QSet<QString> triedBackends;
+    bool foundNonGlobalCoverage = false;
+    for (const auto coverageType : { CoverageArea::Realtime, CoverageArea::Regular, CoverageArea::Any }) {
+        for (const auto &backend : d->m_backends) {
+            if (triedBackends.contains(backend.identifier()) || d->shouldSkipBackend(backend, req)) {
+                continue;
+            }
+            const auto coverage = backend.coverageArea(coverageType);
+            if (coverage.isEmpty() || !coverage.coversLocation(req.location())) {
+                continue;
+            }
+            triedBackends.insert(backend.identifier());
+            foundNonGlobalCoverage |= !coverage.isGlobal();
 
-        auto cache = Cache::lookupLocation(backend->backendId(), req.cacheKey());
-        switch (cache.type) {
-            case CacheHitType::Negative:
-                qCDebug(Log) << "Negative cache hit for backend" << backend->backendId();
-                break;
-            case CacheHitType::Positive:
-                qCDebug(Log) << "Positive cache hit for backend" << backend->backendId();
-                reply->addAttributions(std::move(cache.attributions));
-                reply->addResult(std::move(cache.data));
-                break;
-            case CacheHitType::Miss:
-                qCDebug(Log) << "Cache miss for backend" << backend->backendId();
-                if (backend->queryLocation(req, reply, d->nam())) {
-                    ++pendingOps;
-                }
-                break;
+            reply->addAttribution(BackendPrivate::impl(backend)->attribution());
+
+            auto cache = Cache::lookupLocation(backend.identifier(), req.cacheKey());
+            switch (cache.type) {
+                case CacheHitType::Negative:
+                    qCDebug(Log) << "Negative cache hit for backend" << backend.identifier();
+                    break;
+                case CacheHitType::Positive:
+                    qCDebug(Log) << "Positive cache hit for backend" << backend.identifier();
+                    reply->addAttributions(std::move(cache.attributions));
+                    reply->addResult(std::move(cache.data));
+                    break;
+                case CacheHitType::Miss:
+                    qCDebug(Log) << "Cache miss for backend" << backend.identifier();
+                    if (BackendPrivate::impl(backend)->queryLocation(req, reply, d->nam())) {
+                        ++pendingOps;
+                    }
+                    break;
+            }
+        }
+        if (pendingOps && foundNonGlobalCoverage) {
+            break;
         }
     }
     reply->setPendingOps(pendingOps);
@@ -715,22 +710,22 @@ VehicleLayoutReply* Manager::queryVehicleLayout(const VehicleLayoutRequest &req)
     }
 
     for (const auto &backend : d->m_backends) {
-        if (d->shouldSkipBackend(backend.get(), req)) {
+        if (d->shouldSkipBackend(backend, req)) {
             continue;
         }
-        if (req.stopover().stopPoint().hasCoordinate() && backend->isLocationExcluded(req.stopover().stopPoint())) {
-            qCDebug(Log) << "Skipping backend based on location filter:" << backend->backendId();
+        const auto coverage = backend.coverageArea(CoverageArea::Realtime);
+        if (coverage.isEmpty() || !coverage.coversLocation(req.stopover().stopPoint())) {
             continue;
         }
-        reply->addAttribution(backend->attribution());
+        reply->addAttribution(BackendPrivate::impl(backend)->attribution());
 
-        auto cache = Cache::lookupVehicleLayout(backend->backendId(), req.cacheKey());
+        auto cache = Cache::lookupVehicleLayout(backend.identifier(), req.cacheKey());
         switch (cache.type) {
             case CacheHitType::Negative:
-                qCDebug(Log) << "Negative cache hit for backend" << backend->backendId();
+                qCDebug(Log) << "Negative cache hit for backend" << backend.identifier();
                 break;
             case CacheHitType::Positive:
-                qCDebug(Log) << "Positive cache hit for backend" << backend->backendId();
+                qCDebug(Log) << "Positive cache hit for backend" << backend.identifier();
                 if (cache.data.size() == 1) {
                     reply->addAttributions(std::move(cache.attributions));
                     reply->addResult(cache.data[0]);
@@ -738,8 +733,8 @@ VehicleLayoutReply* Manager::queryVehicleLayout(const VehicleLayoutRequest &req)
                 }
                 Q_FALLTHROUGH();
             case CacheHitType::Miss:
-                qCDebug(Log) << "Cache miss for backend" << backend->backendId();
-                if (backend->queryVehicleLayout(req, reply, d->nam())) {
+                qCDebug(Log) << "Cache miss for backend" << backend.identifier();
+                if (BackendPrivate::impl(backend)->queryVehicleLayout(req, reply, d->nam())) {
                     ++pendingOps;
                 }
                 break;
@@ -767,7 +762,7 @@ QVariantList Manager::attributionsVariant() const
 
 const std::vector<Backend>& Manager::backends() const
 {
-    return d->m_backendMetaData;
+    return d->m_backends;
 }
 
 bool Manager::isBackendEnabled(const QString &backendId) const
